%grammar
    Grammar 
        = "%grammar" ProductionList
          OptPrecedences
          OptMacros
          "%tokens"  TokenList
        ;        
    ProductionList
        = Production
        | ProductionList Production
        ;
    Production
        = NAME "=" SymbolDefinitionList ";"
        ;
    SymbolDefinitionList
        = SymbolDefinition
        | SymbolDefinitionList "|" SymbolDefinition
        ;
    SymbolDefinition
        = OptRuleName OptSymbolList
        ;
    OptRuleName
        =
        | "{" NAME "}"
        ;
    OptSymbolList
        = 
        | SymbolList
        ;
    SymbolList
        = Symbol
        | SymbolList Symbol
        ;
    Symbol
        = { Name   } NAME OptQuant
        | { Text   } TEXT OptQuant
        | { Nested } "(" SymbolList ")" OptQuant 
        ;
    OptPrecedences
    	= "%prec" PrecedenceList
    	|
    	;
    PrecedenceList
        = Precedence
        | PrecedenceList Precedence
        ;
    Precedence
        = PrecedenceSymbolList ":" ASSOC ";"
        ;
    PrecedenceSymbolList
        = PrecedenceSymbol
        | PrecedenceSymbolList "," PrecedenceSymbol
        ;
    PrecedenceSymbol
    	= { Name } NAME
    	| { Text } TEXT
    	;
    OptMacros
    	= "%macros" MacroList
    	|
    	;
    MacroList
        = Macro
        | MacroList Macro
        ;
    Macro
        = NAME "=" RegExp ";"
        ;        
    RegExp
        = { Cat      } RegExp RegExp
        | { Alt      } RegExp "|" RegExp
        | { Nested   } "(" RegExp ")" OptQuant
        | { Macro    } NAME OptQuant
        | { Text     } TEXT OptQuant
        | { Range    } RANGE OptQuant
        | { SubRange } RangeExp "\\" RangeExp
        ;
    RangeExp
        = { Name } NAME
        | { Text } RANGE
        ;
    OptQuant
        =
        | QUANT
        ;
    TokenList
        = Token
        | TokenList Token
        ;
    Token
        = NAME OptStartCondList "=" RegExp OptEventList OptCondShift ";"
        ;
    OptStartCondList
        =
        | "@" StartCondList
        ;
    StartCondList
        = NAME
        | StartCondList "," NAME
        ;
    OptEventList
        =
        | "{" EventList "}"
        ;
    EventList
        = TEXT
        | EventList "," TEXT
        ;
    OptCondShift
        =
        | ">>" NAME
        ;
%prec
    CatRegExp 			: left;
    "("					: right;  
    RANGE, TEXT, NAME 	: none;
    "|"					: left;
%macros
    any = [^] 
        ;
    dot = any \ [\n\r]
        ;
    esc = dot \ [\\]
        ;
    letter
        = [A-Za-z_]
        ;
    digit
        = [0-9]
        ;
      
%tokens
    WS
    	= [ \t]+
            { "Skip" }
        ;
    NL
    	= "\r" | "\n" | "\r\n"
            { "NewLine", "Skip" }
        ;
    QUANT
        = "?" | "*" | "+"
        ;
    ASSOC
        = "left" | "right" | "none"
        ;
    NAME
        = letter (letter | digit)*
        ;
    TEXT
        = "\"" (esc \ ["] | "\\" dot)* "\""
        ;
    RANGE
        = "[" "^"? (esc \ [\]] | "\\" dot)* "]"
        ;
	COMMENT
		= "#"
		  	{ "Skip" } >> X_COMMENT
		;
	COMMENT_LINE @ X_COMMENT
		= dot*
			{ "Skip" } >> MAIN
		;
