/**
 * Beaver: a compiler front-end compiler
 * Copyright (c) 2003-2012, Alexander Demenchuk
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
package beaver.cc.lexer;

import junit.framework.TestCase;

public class CharClassTest extends TestCase {

    public void testNumberScanner() {
        CharSet cset = new CharSet(LexerTestFixture.DigitalScanner());
        assertEquals(2, cset.size);
        CharMap classes = cset.classes;
        assertEquals(14, classes.size());
        CharClass num = (CharClass) classes.get('0');
        assertSame(num, classes.get('1'));
        assertSame(num, classes.get('2'));
        assertSame(num, classes.get('3'));
        assertSame(num, classes.get('4'));
        assertSame(num, classes.get('5'));
        assertSame(num, classes.get('6'));
        assertSame(num, classes.get('7'));
        assertSame(num, classes.get('8'));
        assertSame(num, classes.get('9'));
        assertEquals(10, num.cardinality);
        CharClass spc = (CharClass) classes.get(' ');
        assertSame(spc, classes.get('\t'));
        assertSame(spc, classes.get('\n'));
        assertSame(spc, classes.get('\r'));
        assertEquals(4, spc.cardinality);
        assertNotSame(spc, num);
    }

    public void testDecimalScanner() {
        CharSet cset = new CharSet(LexerTestFixture.DecimalScanner());
        assertEquals(3, cset.size);
        CharMap classes = cset.classes;
        assertEquals(15, classes.size());
        CharClass num = (CharClass) classes.get('0');
        assertSame(num, classes.get('1'));
        assertSame(num, classes.get('2'));
        assertSame(num, classes.get('3'));
        assertSame(num, classes.get('4'));
        assertSame(num, classes.get('5'));
        assertSame(num, classes.get('6'));
        assertSame(num, classes.get('7'));
        assertSame(num, classes.get('8'));
        assertSame(num, classes.get('9'));
        assertEquals(10, num.cardinality);
        CharClass spc = (CharClass) classes.get(' ');
        assertSame(spc, classes.get('\t'));
        assertSame(spc, classes.get('\n'));
        assertSame(spc, classes.get('\r'));
        assertEquals(4, spc.cardinality);
        assertNotSame(spc, num);
        CharClass dot = (CharClass) classes.get('.');
        assertEquals(1, dot.cardinality);
        assertNotSame(spc, dot);
        assertNotSame(num, dot);
    }

    public void testIntegerScanner() {
        CharSet cset = new CharSet(LexerTestFixture.IntegerScanner());
        assertEquals(6, cset.size);
        CharMap classes = cset.classes;
        assertEquals(28, classes.size());
        CharClass nonZeroDigit = (CharClass) classes.get('1');
        assertSame(nonZeroDigit, classes.get('2'));
        assertSame(nonZeroDigit, classes.get('3'));
        assertSame(nonZeroDigit, classes.get('4'));
        assertSame(nonZeroDigit, classes.get('5'));
        assertSame(nonZeroDigit, classes.get('6'));
        assertSame(nonZeroDigit, classes.get('7'));
        assertSame(nonZeroDigit, classes.get('8'));
        assertSame(nonZeroDigit, classes.get('9'));
        assertEquals(9, nonZeroDigit.cardinality);
        CharClass spc = (CharClass) classes.get(' ');
        assertSame(spc, classes.get('\t'));
        assertSame(spc, classes.get('\n'));
        assertSame(spc, classes.get('\r'));
        assertEquals(4, spc.cardinality);
        assertNotSame(spc, nonZeroDigit);
        CharClass zero = (CharClass) classes.get('0');
        assertEquals(1, zero.cardinality);
        assertNotSame(spc, zero);
        assertNotSame(nonZeroDigit, zero);
        CharClass hex = (CharClass) classes.get('x');
        assertEquals(1, hex.cardinality);
        assertNotSame(spc, hex);
        assertNotSame(nonZeroDigit, hex);
        assertNotSame(zero, hex);
        CharClass letter = (CharClass) classes.get('A');
        assertSame(letter, classes.get('B'));
        assertSame(letter, classes.get('C'));
        assertSame(letter, classes.get('D'));
        assertSame(letter, classes.get('E'));
        assertSame(letter, classes.get('F'));
        assertSame(letter, classes.get('a'));
        assertSame(letter, classes.get('b'));
        assertSame(letter, classes.get('c'));
        assertSame(letter, classes.get('d'));
        assertSame(letter, classes.get('e'));
        assertSame(letter, classes.get('f'));
        assertEquals(12, letter.cardinality);
        assertNotSame(spc, letter);
        assertNotSame(nonZeroDigit, letter);
        assertNotSame(zero, letter);
        assertNotSame(hex, letter);
    }
}
