/**
 * Beaver: a compiler front-end compiler
 * Copyright (c) 2003-2012, Alexander Demenchuk
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
package beaver.cc.lexer;

import junit.framework.TestCase;

public class CharMapTest extends TestCase {

    public void testPutAndGet() {
        CharMap map = new CharMap();
        map.put('a', "a");
        assertEquals("a", map.get('a'));
        map.put('b', "b");
        assertEquals("b", map.get('b'));
        assertNull(map.get('c'));
    }

    public void testClear() {
        CharMap map = new CharMap();
        map.put('a', "a");
        map.put('b', "b");
        map.clear();
        assertNull(map.get('a'));
        assertNull(map.get('b'));
    }

    public void testVisitor() {
        CharMap map = new CharMap();
        map.put('a', "a");
        map.put('b', "b");
        final int[] counters = new int[1];
        map.accept(new CharMap.Visitor() {
            public void visit(char key, Object value) {
                String v = value.toString();
                assertEquals(1, v.length());
                assertEquals(key, value.toString().charAt(0));
                counters[0]++;
            }
        });
        assertEquals(2, counters[0]);
    }

    public void testSize() {
        CharMap map = new CharMap();
        assertEquals(0, map.size());
        map.put('a', "a");
        assertEquals(1, map.size());
        map.put('b', "b");
        assertEquals(2, map.size());
        map.put('c', "b");
        assertEquals(3, map.size());
        map.clear();
        assertEquals(0, map.size());
    }
}
