/**
 * Beaver: a compiler front-end compiler
 * Copyright (c) 2003-2012, Alexander Demenchuk
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
package beaver.cc.lexer;

import junit.framework.TestCase;

public class CharReaderTest extends TestCase {

    public void testTrim() {
        CharReader str = new CharReader("abcdef");
        str.trim(3);
        assertTrue(str.isEmpty());
    }

    public void testTrim1() {
        CharReader str = new CharReader("abcdef");
        str.trim(1);
        str.trim(1);
        str.trim(1);
        assertTrue(str.isEmpty());
    }

    public void testIsEmpty() {
        CharReader str = new CharReader("");
        assertTrue(str.isEmpty());
    }

    public void testReadChar() {
        CharReader str = new CharReader("ab\\t\\x63\\u0064");
        assertEquals('a', (char) str.readChar());
        assertEquals('b', (char) str.readChar());
        assertEquals('\t', (char) str.readChar());
        assertEquals('c', (char) str.readChar());
        assertEquals('d', (char) str.readChar());
    }

    public void testReadDigit() {
        CharReader str = new CharReader("25AF");
        assertEquals(2, str.readDigit(10));
        assertEquals(5, str.readDigit(10));
        assertEquals(10, str.readDigit(16));
        assertEquals(15, str.readDigit(16));
    }

    public void testReadText() {
        CharReader str = new CharReader("\\t123.456\\r\\n567.89 098.48\\b");
        assertEquals('\t', (char) str.readChar());
        assertEquals('1', (char) str.readChar());
        assertEquals('2', (char) str.readChar());
        assertEquals('3', (char) str.readChar());
        assertEquals('.', (char) str.readChar());
        assertEquals('4', (char) str.readChar());
        assertEquals('5', (char) str.readChar());
        assertEquals('6', (char) str.readChar());
        assertEquals('\r', (char) str.readChar());
        assertEquals('\n', (char) str.readChar());
        assertEquals('5', (char) str.readChar());
        assertEquals('6', (char) str.readChar());
        assertEquals('7', (char) str.readChar());
        assertEquals('.', (char) str.readChar());
        assertEquals('8', (char) str.readChar());
        assertEquals('9', (char) str.readChar());
        assertEquals(' ', (char) str.readChar());
        assertEquals('0', (char) str.readChar());
        assertEquals('9', (char) str.readChar());
        assertEquals('8', (char) str.readChar());
        assertEquals('.', (char) str.readChar());
        assertEquals('4', (char) str.readChar());
        assertEquals('8', (char) str.readChar());
        assertEquals('\b', (char) str.readChar());
        assertEquals(-1, str.readChar());

    }
}
