/**
 * Beaver: a compiler front-end compiler
 * Copyright (c) 2003-2012, Alexander Demenchuk
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
package beaver.cc.lexer;

import junit.framework.TestCase;

public class NFATest extends TestCase {

    public void testDecimalScanner() {
        NFA nfa = new NFA(LexerTestFixture.DecimalScanner());

        NFA.Node start = nfa.start;
        assertTrue(start instanceof NFA.Node.Fork);

        NFA.Node.Fork n0 = (NFA.Node.Fork) start;

        assertTrue(n0.next instanceof NFA.Node.Char);
        NFA.Node.Char n1 = (NFA.Node.Char) n0.next;
        assertEquals(1, n1.charClasses.length);
        assertSame(nfa.cset.classes.get('0'), n1.charClasses[0]);

        assertTrue(n1.next instanceof NFA.Node.Fork);
        NFA.Node.Fork n2 = (NFA.Node.Fork) n1.next;

        assertTrue(n2.alt instanceof NFA.Node.Char);
        NFA.Node.Char n3 = (NFA.Node.Char) n2.alt;
        assertEquals(1, n3.charClasses.length);
        assertSame(nfa.cset.classes.get('0'), n3.charClasses[0]);
        assertSame(n2, n3.next);

        assertTrue(n2.next instanceof NFA.Node.Fork);
        NFA.Node.Fork n4 = (NFA.Node.Fork) n2.next;

        assertTrue(n4.next instanceof NFA.Node.Char);
        NFA.Node.Char n5 = (NFA.Node.Char) n4.next;
        assertEquals(1, n5.charClasses.length);
        assertSame(nfa.cset.classes.get('.'), n5.charClasses[0]);

        assertTrue(n5.next instanceof NFA.Node.Char);
        NFA.Node.Char n6 = (NFA.Node.Char) n5.next;
        assertEquals(1, n6.charClasses.length);
        assertSame(nfa.cset.classes.get('0'), n6.charClasses[0]);

        assertTrue(n6.next instanceof NFA.Node.Fork);
        NFA.Node.Fork n7 = (NFA.Node.Fork) n6.next;

        assertTrue(n7.alt instanceof NFA.Node.Char);
        NFA.Node.Char n8 = (NFA.Node.Char) n7.alt;
        assertEquals(1, n8.charClasses.length);
        assertSame(nfa.cset.classes.get('0'), n8.charClasses[0]);
        assertSame(n7, n8.next);

        assertTrue(n7.next instanceof NFA.Node.Term);
        NFA.Node.Term n9 = (NFA.Node.Term) n7.next;
        assertSame(n9, n4.alt);
        assertEquals(1, n9.patternMatchResult.id);
        assertNull(n9.next);

        assertTrue(n0.alt instanceof NFA.Node.Char);
        NFA.Node.Char n10 = (NFA.Node.Char) n0.alt;
        assertEquals(1, n10.charClasses.length);
        assertSame(nfa.cset.classes.get(' '), n10.charClasses[0]);
        assertTrue(n10.next instanceof NFA.Node.Term);

        NFA.Node.Term n11 = (NFA.Node.Term) n10.next;
        assertEquals(2, n11.patternMatchResult.id);
        assertNull(n11.next);
    }
}
